@@ -563,7 +563,7 @@ uInt32 DSTCPEndpoint::WriteData ( const void *inData, const uInt32 inSize )
 			//TODO do we need a socket level timeout here ie. setsocketopt with SO_SNDTIMEO
 			do
 			{
-				rc = ::sendto(mConnectFD, aPtr, dataSize, NULL, NULL, NULL);
+				rc = ::sendto(mConnectFD, aPtr, dataSize, 0, NULL, 0);
 				if (mAborting == true)
 				{
 					throw((sInt32)kAbortedWarning);
@@ -835,11 +835,11 @@ int DSTCPEndpoint::DoTCPListen ( void )
 
 int DSTCPEndpoint::DoTCPAccept ( void )
 {
-	int		err		= eDSNoErr;
-	int		aLen	= sizeof( mRemoteSockAddr );
-	int		rc		= eDSNoErr;
-	fd_set	readSet;
-
+	int			err		= eDSNoErr;
+	socklen_t	aLen	= sizeof( mRemoteSockAddr );
+	int			rc		= eDSNoErr;
+	fd_set		readSet;
+	
 	do {
 		FD_ZERO( &readSet );
 		FD_SET( mListenFD, &readSet );
@@ -869,7 +869,7 @@ int DSTCPEndpoint::DoTCPAccept ( void )
 		}
 	} while ( !FD_ISSET( mListenFD, &readSet ) );
 
-	mConnectFD = ::accept( mListenFD, (struct sockaddr *)&mRemoteSockAddr, &aLen );
+	mConnectFD = ::accept( mListenFD, (struct sockaddr *)&mRemoteSockAddr, (socklen_t*)&aLen );
 
 	if ( mAborting == true )
 	{
@@ -1079,6 +1079,7 @@ uInt32 DSTCPEndpoint::DoTCPRecvFrom ( void *ioBuffer, const uInt32 inBufferSize
 void * DSTCPEndpoint::GetClientMessage ( void )
 {
 	sComData			   *pOutMsg			= nil;
+	sComProxyData		   *pOutProxyMsg	= nil;
 	void				   *tmpOutMsg		= nil;
 	uInt32					buffLen			= 0;
 	uInt32					readBytes		= 0;
@@ -1113,27 +1114,27 @@ void * DSTCPEndpoint::GetClientMessage ( void )
 				else
 				{
 					DecryptData(inBuffer, inLength, tmpOutMsg, buffLen);
-					pOutMsg = (sComData *) tmpOutMsg;
+					pOutProxyMsg = (sComProxyData *) tmpOutMsg;
 					if (buffLen == 0)
 					{
-						pOutMsg		= (sComData *)inBuffer;
+						pOutProxyMsg= (sComProxyData *)inBuffer;
 						inBuffer 	= nil;
 						buffLen		= inLength;
 					}
-					if (pOutMsg != nil)
+					if (pOutProxyMsg != nil)
 					{
-						if (NXSwapBigLongToHost(pOutMsg->fDataSize) > buffLen - sizeof(sComData))
+						if (NXSwapBigLongToHost(pOutProxyMsg->fDataSize) > buffLen - sizeof(sComProxyData))
 						{
 							//fprintf(stderr,"bad message fDataSize!\n");
 							//let's just throw the message out since it is probably malformed
-							free(pOutMsg);
-							pOutMsg = nil;
+							free(pOutProxyMsg);
+							pOutProxyMsg = nil;
 						}
 						//else
 						//{
-							//place the endpoint handle into the pOutMsg struct
+							//place the endpoint handle into the pOutProxyMsg struct
 							//don't create a duplicate
-							//pOutMsg->fPort = (uInt32) this; //don't need this since using direct dispatch
+							//pOutProxyMsg->fPort = (uInt32) this; //don't need this since using direct dispatch
 							//KW use of this endpoint needs to be mutex protected?
 							//not likely since we force a single thread on the open API connection
 						//}
@@ -1142,10 +1143,10 @@ void * DSTCPEndpoint::GetClientMessage ( void )
 			}
 			catch( sInt32 err )
 			{
-				if (pOutMsg != nil)
+				if (pOutProxyMsg != nil)
 				{
-					free(pOutMsg);
-					pOutMsg = nil;
+					free(pOutProxyMsg);
+					pOutProxyMsg = nil;
 				}
 				siResult = eDSTCPReceiveError; //not actually used
 			}
@@ -1153,10 +1154,18 @@ void * DSTCPEndpoint::GetClientMessage ( void )
 			inBuffer = nil;
 		}//if (inBuffer != nil)
 	}
-	
-	DSTCPEndian swapper(pOutMsg, DSTCPEndian::kSwapToHost);
+#ifndef __BIG_ENDIAN__
+	DSTCPEndian swapper(pOutProxyMsg, kDSSwapToHost);
+	swapper.AddIPAndPort( mRemoteHostIPAddr, ntohs( mRemoteSockAddr.sin_port ));
     swapper.SwapMessage();
+#endif
     
+	pOutMsg = AllocFromProxyStruct( pOutProxyMsg );
+	if (pOutProxyMsg != nil)
+	{
+		free(pOutProxyMsg);
+		pOutProxyMsg = nil;
+	}
     return( pOutMsg );
 
 } // GetClientMessage
@@ -1310,12 +1319,24 @@ sInt32 DSTCPEndpoint::SyncToMessageBody(const Boolean inStripLeadZeroes, uInt32
 
 sInt32 DSTCPEndpoint::SendClientReply ( void *inMsg )
 {
-	uInt32 messageSize = sizeof(sComData) + ((sComData *)inMsg)->fDataLength;
+	uInt32                  messageSize = 0;
+	sComProxyData  *inProxyMsg  = nil;
+	sInt32                  sendResult  = eDSNoErr;
+
+	inProxyMsg = AllocToProxyStruct( (sComData *)inMsg );
 	//let us only send the data that is present and not the entire buffer
-	((sComData *)inMsg)->fDataSize = ((sComData *)inMsg)->fDataLength;
-	DSTCPEndian swapper((sComData *)inMsg, DSTCPEndian::kSwapToBig);
-    swapper.SwapMessage();
-    return SendBuffer(inMsg, messageSize);
+	inProxyMsg->fDataSize = inProxyMsg->fDataLength;
+	messageSize = sizeof(sComProxyData) + inProxyMsg->fDataLength;
+#ifndef __BIG_ENDIAN__
+	DSTCPEndian swapper(inProxyMsg, kDSSwapToBig);
+	swapper.AddIPAndPort( mRemoteHostIPAddr, ntohs( mRemoteSockAddr.sin_port ));
+	swapper.SwapMessage();
+#endif
+	sendResult = SendBuffer(inProxyMsg, messageSize);
+	free(inProxyMsg);
+	inProxyMsg = nil;
+	
+	return(sendResult);
 } // SendClientReply
 
 
@@ -1326,12 +1347,23 @@ sInt32 DSTCPEndpoint::SendClientReply ( void *inMsg )
 
 sInt32 DSTCPEndpoint::SendServerMessage ( void *inMsg )
 {
-	uInt32 messageSize = sizeof(sComData) + ((sComData *)inMsg)->fDataLength;
+	uInt32                  messageSize = 0;
+	sComProxyData  *inProxyMsg  = nil;
+	sInt32                  sendResult  = eDSNoErr;
+
+	inProxyMsg = AllocToProxyStruct( (sComData *)inMsg );
 	//let us only send the data that is present and not the entire buffer
-	((sComData *)inMsg)->fDataSize = ((sComData *)inMsg)->fDataLength;
-	DSTCPEndian swapper((sComData *)inMsg, DSTCPEndian::kSwapToBig);
-    swapper.SwapMessage();
-    return SendBuffer(inMsg, messageSize);
+	inProxyMsg->fDataSize = inProxyMsg->fDataLength;
+	messageSize = sizeof(sComProxyData) + inProxyMsg->fDataLength;
+#ifndef __BIG_ENDIAN__
+	DSTCPEndian swapper(inProxyMsg, kDSSwapToBig);
+	swapper.SwapMessage();
+#endif
+	sendResult = SendBuffer(inProxyMsg, messageSize);
+	free(inProxyMsg);
+	inProxyMsg = nil;
+	
+	return(sendResult);
 } // SendServerMessage
 
 
@@ -1425,6 +1457,7 @@ sInt32 DSTCPEndpoint::GetServerReply ( sComData **outMsg )
 	uInt32					readBytes 		= 0;
 	void				   *inBuffer		= nil;
 	uInt32					inLength		= 0;
+	sComProxyData		   *outProxyMsg		= nil;
 
 	//need to read a tag and then a buffer length
 	siResult = SyncToMessageBody(true, &inLength);
@@ -1447,11 +1480,11 @@ sInt32 DSTCPEndpoint::GetServerReply ( sComData **outMsg )
 			{
 				void *tmpOutMsg = nil;
 				DecryptData(inBuffer, inLength, tmpOutMsg, buffLen);
-				*outMsg = (sComData *)tmpOutMsg;
+				outProxyMsg = (sComProxyData *)tmpOutMsg;
 				if (buffLen == 0)
 				{
-					free(*outMsg);
-					*outMsg		= (sComData *)inBuffer;
+					free(outProxyMsg);
+					outProxyMsg	= (sComProxyData *)inBuffer;
 					inBuffer	= nil;
 					buffLen		= inLength;
 				}
@@ -1469,10 +1502,15 @@ sInt32 DSTCPEndpoint::GetServerReply ( sComData **outMsg )
 		inBuffer = nil;
 	}
 	
-    if (*outMsg != nil)
+    if (outProxyMsg != nil)
     {
-        DSTCPEndian swapper(*outMsg, DSTCPEndian::kSwapToHost);
+#ifndef __BIG_ENDIAN__
+        DSTCPEndian swapper(outProxyMsg, kDSSwapToHost);
         swapper.SwapMessage();
+#endif
+		*outMsg = AllocFromProxyStruct( outProxyMsg );
+		free(outProxyMsg);
+		outProxyMsg = nil;
     }
 
 	return( siResult );
@@ -1499,3 +1537,72 @@ uInt16 DSTCPEndpoint::GetRemoteHostPort ( void )
 {
 	return ( ntohs( mRemoteSockAddr.sin_port ) );
 }
+
+//------------------------------------------------------------------------------
+//     * AllocToProxyStruct
+//
+//------------------------------------------------------------------------------
+
+sComProxyData* DSTCPEndpoint::AllocToProxyStruct ( sComData *inDataMsg )
+{
+	sComProxyData      *outProxyDataMsg = nil;
+	int					objIndex;
+
+	if (inDataMsg != nil)
+	{
+		outProxyDataMsg = (sComProxyData *)calloc( 1, sizeof(sComProxyData) + inDataMsg->fDataSize );
+		
+		// this is copying the head data from sComProxyData to sComData
+		bcopy( inDataMsg, outProxyDataMsg, (char *)(outProxyDataMsg->obj) - (char *)outProxyDataMsg );
+
+		// this copies the sObject and the actual data
+		bcopy( inDataMsg->obj, outProxyDataMsg->obj, kObjSize + inDataMsg->fDataSize );
+		
+		//need to adjust the offsets since they are relative to the start of the message
+		for ( objIndex = 0; objIndex < 10; objIndex++ )
+		{
+			if ( outProxyDataMsg->obj[ objIndex ].offset != 0 )
+			{
+				outProxyDataMsg->obj[ objIndex ].offset -= sizeof(sComData) - sizeof(sComProxyData);
+			}
+		}
+	}
+
+	return ( outProxyDataMsg );
+}
+
+//------------------------------------------------------------------------------
+//     * AllocFromProxyStruct
+//
+//------------------------------------------------------------------------------
+
+sComData* DSTCPEndpoint::AllocFromProxyStruct ( sComProxyData *inProxyDataMsg )
+{
+	sComData                   *outDataMsg          = nil;
+	int							objIndex;
+
+	if (inProxyDataMsg != nil)
+	{
+		outDataMsg = (sComData *)calloc( 1, sizeof(sComData) + inProxyDataMsg->fDataSize );
+		
+		// this is copying the head data from sComProxyData to sComData
+		bcopy( inProxyDataMsg, outDataMsg, (char *)(inProxyDataMsg->obj) - (char *)inProxyDataMsg );
+		
+		// this copies the sObject and the actual data
+		bcopy( inProxyDataMsg->obj, outDataMsg->obj, kObjSize + inProxyDataMsg->fDataSize );
+		
+		//need to adjust the offsets since they are relative to the start of the message
+		for ( objIndex = 0; objIndex < 10; objIndex++ )
+		{
+			if ( outDataMsg->obj[ objIndex ].offset != 0 )
+			{
+				outDataMsg->obj[ objIndex ].offset += sizeof(sComData) - sizeof(sComProxyData);
+			}
+		}
+		
+		// set the effective UIDs to -2...
+		outDataMsg->fUID = outDataMsg->fEffectiveUID = (uid_t) -2;
+	}
+
+	return ( outDataMsg );
+}
